/* 
  Crazy Crane
  Grzegorz Kraszewski, 2013.
  Time counter routines.
*/

#include "main.h"
#include "counter.h"

#include <proto/exec.h>
#include <proto/timer.h>
#include <proto/utility.h>


#ifdef TARGET_AMIGAOS3

ULONG GetTick(ULONG efreq)
{
	return UDivMod32(efreq, 50);
}

#else

ULONG GetTick(ULONG efreq)
{
	return efreq / 50;
}

#endif


void AddEClock(struct EClockVal *dest, ULONG add)
{
	ULONG a;

	a = dest->ev_lo + add;
	if (a < dest->ev_lo) dest->ev_hi++;
	dest->ev_lo = a;
}



void CounterDispose(struct Counter *c)
{
	if (c->Port)
	{
		if (c->Req)
		{
			AbortIO((struct IORequest*)c->Req);
			WaitIO((struct IORequest*)c->Req);
			#ifdef TARGET_AMIGAOS4
			DropInterface((struct Interface*)c->ITimer);
			#endif
			CloseDevice((struct IORequest*)c->Req);
			DeleteIORequest((struct IORequest*)c->Req);
		}

		DeleteMsgPort(c->Port);
	}
}



BOOL CounterNew(struct Counter *c)
{
	struct Library *TimerBase;
	#ifdef TARGET_AMIGAOS4
	struct TimerIFace *ITimer;
	#endif

	if (c->Port = CreateMsgPort())
	{
		if (c->Req = (struct TimeRequest*)CreateIORequest(c->Port, sizeof(struct TimeRequest)))
		{
			if (OpenDevice("timer.device", UNIT_WAITECLOCK, (struct IORequest*)c->Req, 0) == 0)
			{
				TimerBase = &c->Req->Request.io_Device->dd_Library;
				#ifdef TARGET_AMIGAOS4
				if (ITimer = (struct TimerIFace*)GetInterface(TimerBase, "main", 1, NULL))
				{
					c->ITimer = ITimer;
					#endif
					c->TimerBase = TimerBase;
					c->EClockFreq = ReadEClock(&c->NextTick);
					c->ETick = GetTick(c->EClockFreq);
					AddEClock(&c->NextTick, c->ETick);
					c->Req->Request.io_Command = TR_ADDREQUEST;
					c->Req->Time.Seconds = c->NextTick.ev_hi;
					c->Req->Time.Microseconds = c->NextTick.ev_lo;
					SendIO((struct IORequest*)c->Req);
					return TRUE;
					#ifdef TARGET_AMIGAOS4
				}
				#endif
			}
		}
	}

	CounterDispose(c);
	return FALSE;
}



void CounterRetrig(struct Counter *c)
{
	struct Library *TimerBase;
	#ifdef TARGET_AMIGAOS4
	struct TimerIFace *ITimer = c->ITimer;
	#endif

	TimerBase = c->TimerBase;
	GetMsg(c->Port);
	AddEClock(&c->NextTick, c->ETick);
	c->Req->Request.io_Command = TR_ADDREQUEST;
	c->Req->Time.Seconds = c->NextTick.ev_hi;
	c->Req->Time.Microseconds = c->NextTick.ev_lo;
	SendIO((struct IORequest*)c->Req);
}
