/* Crazy Crane */
/* Main code file. */

#include "main.h"
#include "gfx.h"

#include <proto/graphics.h>
#include <proto/layers.h>
#include <proto/intuition.h>

extern struct GameData Gd;

/* Let's use datatypes on AmigaOS, Reggae on MorphOS */

#ifdef TARGET_MORPHOS
#include <proto/multimedia.h>
#include <classes/multimedia/video.h>
#endif  /* TARGET_MORPHOS */

#ifdef TARGET_AMIGAOS4
#define MBMF_MINPLANES 0
#include <graphics/blitattr.h>
#else
#define MBMF_MINPLANES BMF_MINPLANES
#include <proto/cybergraphics.h>
#include <cybergraphx/cybergraphics.h>
#include <clib/alib_protos.h>
#endif


BOOL GraphicsLoadA(char *filepath);
#ifndef TARGET_MORPHOS
BOOL GraphicsLoadB(Object *image, UWORD width, UWORD height);
#endif
#ifdef TARGET_AMIGAOS3
BOOL GraphicsLoadC(Object *image, UWORD width, UWORD height);
#endif




/* Table for ISO-8859-2 */

const char* Letters = "000041A14243C64445CA464748494A4B4CA34D4ED14FD350515253A65455565758595AACAF0000";

//==============================================================================================================================
// This layer hook blits background bitmap into rectangles damaged by scrolling.

struct LayerHookMsg
{
	struct Layer *Layer;
	struct Rectangle Bounds;
	LONG OffsetX;
	LONG OffseyY;
};


#ifdef TARGET_MORPHOS

IPTR LayerHookFunc(void)
{
	struct Hook *h = (struct Hook*)REG_A0;
	struct RastPort *rp = (struct RastPort*)REG_A2;
	struct LayerHookMsg *lm = (struct LayerHookMsg*)REG_A1;

#endif
#ifdef TARGET_AMIGAOS3

__saveds IPTR LayerHookFunc(struct Hook *h __asm("a0"), struct LayerHookMsg *lm __asm("a1"), struct RastPort *rp __asm("a2"))
{

#endif
#ifdef TARGET_AMIGAOS4

VARARGS68K IPTR LayerHookFunc(struct Hook *h, struct RastPort *rp, struct LayerHookMsg *lm)
{

#endif

	struct GameData *gd = (struct GameData*)h->h_Data;
	UWORD xo, yo, wo, ho;

	xo = (UWORD)lm->OffsetX - gd->GameWin->BorderLeft;
	yo = (UWORD)lm->OffseyY - gd->GameWin->BorderTop + gd->BgOffset;
	wo = lm->Bounds.MaxX - lm->Bounds.MinX + 1;
	ho = lm->Bounds.MaxY - lm->Bounds.MinY + 1;
	BltBitMap(gd->GameGfx, xo, yo, rp->BitMap, lm->Bounds.MinX, lm->Bounds.MinY, wo, ho, 0xC0, ~0, NULL);
	return 0;
}


#ifdef TARGET_MORPHOS

struct EmulLibEntry LayerHookGate = {TRAP_LIB, 0, (void(*)(void))LayerHookFunc};
struct Hook LayerHook = {{0, 0}, (HOOKFUNC)&LayerHookGate, 0, &Gd};

#else

struct Hook LayerHook = {{0, 0}, (HOOKFUNC)LayerHookFunc, 0, &Gd};

#endif



//==============================================================================================================================
// This function assumes only uppercase hexdigits.

UBYTE HexToBin(char hex)
{
	UBYTE x = 0;

	if ((hex >= '0') && (hex <= '9')) x = hex - 48;
	else if ((hex >= 'A') && (hex <= 'F')) x = hex - 55;
	return x;
}

//==============================================================================================================================

void CreateLetterTable(void)
{
	LONG i, x;

	for (i = 0; i < 39; i++)
	{
		x = HexToBin(*Letters++) << 4;
		x |= HexToBin(*Letters++);
		Gd.LetterTable[i] = x;
	}
}

//==============================================================================================================================

WORD GetLetterX(UBYTE letter)
{
	WORD i, x = 0, f;

	switch (letter)
	{
		case ' ':                                              // background
		case '-':   return 0;                                  // bar, internal segment
		case '[':                                              // bar, left edge
		case ']':   return Gd.BlockWidth;                      // bar, right edge
		default:
			f = 0;

			for (i = 0; i < 39; i++)
			{
				if (Gd.LetterTable[i] == letter) return f;
				f += Gd.BlockWidth;
			}
		return x;
	}
}

//==============================================================================================================================

WORD GetLetterY(UBYTE letter, BOOL pressed)
{
	switch (letter)
	{
		case ' ':                                              // background
		case ']':   return 0;                                  // bar, right edge
		case '[':                                              // bar, left edge
		case '-':   return Gd.BlockHeight;                     // bar, internal segment
		default:    return pressed ? Gd.BlockHeight : 0;       // letters
	}
}

//==============================================================================================================================

BOOL GraphicsLoad(char *filepath)
{
	BOOL success = FALSE;

	Gd.GameScr = LockPubScreen(NULL);
	CreateLetterTable();
	success = GraphicsLoadA(filepath);
	return success;
}


//==============================================================================================================================

BOOL GraphicsDimensions(UWORD width, UWORD height)
{
	if ((width % 39 == 0) && (height % 36 == 0))
	{
		Gd.BlockWidth = width / 39;
		Gd.BlockHeight = height / 36;
		Gd.BgOffset = Gd.BlockHeight * 19;
		return TRUE;
	}

	return FALSE;
}


//==============================================================================================================================

#ifndef TARGET_MORPHOS

#include <proto/datatypes.h>
#include <datatypes/pictureclass.h>

BOOL GraphicsLoadA(char *filepath)
{
	BOOL success = FALSE;
	Object *image;

	if (image = NewDTObject(filepath,
		DTA_GroupID, GID_PICTURE,
		PDTA_DestMode, PMODE_V43,
	TAG_END))
	{
		struct BitMapHeader *bmh;

		GetDTAttrs(image, PDTA_BitMapHeader, (IPTR)&bmh, TAG_END);

		if (bmh)
		{
			UWORD width, height;

			width = bmh->bmh_Width;
			height = bmh->bmh_Height;

			if (GraphicsDimensions(width, height))
			{
				#ifdef TARGET_AMIGAOS4

				success = GraphicsLoadB(image, width, height);

				#else

				ULONG screen_depth = GetBitMapAttr(Gd.GameScr->RastPort.BitMap, BMA_DEPTH);

				if (screen_depth <= 8) success = GraphicsLoadC(image, width, height);
				else success = GraphicsLoadB(image, width, height);

				#endif
			}

			DisposeDTObject(image);
		}
	}

	return success;
}


//==============================================================================================================================


BOOL GraphicsLoadB(Object *image, UWORD width, UWORD height)
{
	BOOL success = FALSE;

	if (Gd.GameGfx = AllocBitMap(width, height, 32, MBMF_MINPLANES | BMF_CLEAR, Gd.GameScr->RastPort.BitMap))
	{
		ULONG argb32_size, *argb32_data;

		argb32_size = width * height << 2;

		if (argb32_data = AllocMem(argb32_size, MEMF_ANY))
		{
			#ifdef TARGET_AMIGAOS4

			DoMethod(image, PDTM_READPIXELARRAY, (IPTR)argb32_data, PBPAFMT_ARGB, width << 2, 0, 0, width, height);

			BltBitMapTags(
				BLITA_Source, argb32_data,
				BLITA_SrcType, BLITT_ARGB32,
				BLITA_SrcX, 0,
				BLITA_SrcY, 0,
				BLITA_SrcBytesPerRow, width << 2,
				BLITA_Dest, Gd.GameGfx,
				BLITA_DestType, BLITT_BITMAP,
				BLITA_DestX, 0,
				BLITA_DestY, 0,
				BLITA_Width, width,
				BLITA_Height, height,
			TAG_END);

			#else

			struct RastPort trp;
			struct Library *CyberGfxBase;

			if (CyberGfxBase = OpenLibrary("cybergraphics.library", 0))
			{
				InitRastPort(&trp);
				trp.BitMap = Gd.GameGfx;
				DoMethod(image, PDTM_READPIXELARRAY, (IPTR)argb32_data, PBPAFMT_ARGB, width << 2, 0, 0, width, height);
				WritePixelArray(argb32_data, 0, 0, width << 2, &trp, 0, 0, width, height, RECTFMT_ARGB);
				CloseLibrary(CyberGfxBase);
			}

			#endif

			if (Gd.ScratchPad = AllocBitMap(Gd.BlockWidth * 28, Gd.BlockHeight * 6, 32, BMF_DISPLAYABLE | MBMF_MINPLANES,
			 Gd.GameScr->RastPort.BitMap))
			{
				success = TRUE;
			}

			FreeMem(argb32_data, argb32_size);
		}
	}

	return success;
}


//==============================================================================================================================

#ifdef TARGET_AMIGAOS3

BOOL GraphicsLoadC(Object *image, UWORD width, UWORD height)
{
	BOOL success = FALSE;
	struct BitMap *dtbm;

	SetDTAttrs(image, NULL, NULL,
		PDTA_Screen, (IPTR)Gd.GameScr,
		PDTA_Remap, TRUE,
	TAG_END);

	DoDTMethod(image, NULL, NULL, DTM_PROCLAYOUT);
	GetDTAttrs(image, PDTA_DestBitMap, (IPTR)&dtbm, TAG_END);

	if (dtbm)
	{
		LONG depth;

		depth = GetBitMapAttr(dtbm, BMA_DEPTH);

		if (Gd.GameGfx = AllocBitMap(width, height, depth, 0, Gd.GameScr->RastPort.BitMap))
		{
			struct RastPort trp;

			InitRastPort(&trp);
			trp.BitMap = Gd.GameGfx;
			BltBitMapRastPort(dtbm, 0, 0, &trp, 0, 0, width, height, 0xC0);

			if (Gd.ScratchPad = AllocBitMap(Gd.BlockWidth * 28, Gd.BlockHeight * 6, depth, BMF_DISPLAYABLE | BMF_MINPLANES,
			 Gd.GameScr->RastPort.BitMap))
			{
				success = TRUE;
			}
		}
	}

	return success;
}

#endif /* TARGET_AMIGAOS3 */

#else  /* !TARGET_MORPHOS */

#include <proto/multimedia.h>
#include <proto/cybergraphics.h>
#include <classes/multimedia/video.h>
#include <cybergraphx/cybergraphics.h>

BOOL GraphicsLoadA(char *filepath)
{
	BOOL success = FALSE;
	Object *image;

	if (image = MediaNewObjectTags(
		MMA_StreamName, (IPTR)filepath,
		MMA_StreamType, (IPTR)"file.stream",
		MMA_MediaType, MMT_PICTURE,
	))
	{
		UWORD width, height;
		ULONG tmpsize;
		ULONG *temp_array;

		width = MediaGetPort(image, 0, MMA_Video_Width);
		height = MediaGetPort(image, 0, MMA_Video_Height);
		tmpsize = width * height << 2;

		if (GraphicsDimensions(width, height))
		{
			if (temp_array = MediaAllocVec(tmpsize))
			{
				DoMethod(image, MMM_Pull, 0, (IPTR)temp_array, tmpsize);

				if (Gd.GameGfx = AllocBitMap(width, height, 32, BMF_MINPLANES, Gd.GameScr->RastPort.BitMap))
				{
					struct RastPort trp;

					InitRastPort(&trp);
					trp.BitMap = Gd.GameGfx;
					WritePixelArray(temp_array, 0, 0, width << 2, &trp, 0, 0, width, height, RECTFMT_ARGB);

					if (Gd.ScratchPad = AllocBitMap(Gd.BlockWidth * 28, Gd.BlockHeight * 6, 32, BMF_DISPLAYABLE | BMF_MINPLANES,
					 Gd.GameScr->RastPort.BitMap))
					{
						success = TRUE;
					}
				}

				MediaFreeVec(temp_array);
			}
		}

		DisposeObject(image);
	}

	return success;
}

#endif  /* !TARGET_MORPHOS */

//==============================================================================================================================

void GraphicsDispose(void)
{
	if (Gd.GameScr) UnlockPubScreen(NULL, Gd.GameScr);
	if (Gd.ScratchPad) FreeBitMap(Gd.ScratchPad);
	if (Gd.GameGfx) FreeBitMap(Gd.GameGfx);
}

//==============================================================================================================================
// Blits a (w, h) rectangle from (src_x, src_y) in the game graphics bitmap to (dst_x, dst_y) in the game window, adding window
// top and left border. This routine has separate versions for every target. MorphOS and AmigaOS 4 versions support source alpha
// channel.

#ifdef TARGET_MORPHOS

void BlitRect(UWORD src_x, UWORD src_y, UWORD dst_x, UWORD dst_y, UWORD w, UWORD h)
{
	struct TagItem tags[] = {
		{ BLTBMA_USESOURCEALPHA, TRUE },
		{ TAG_END, 0 }
	};

	dst_x += Gd.GameWin->BorderLeft;
	dst_y += Gd.GameWin->BorderTop;

	BltBitMapRastPortAlpha(Gd.GameGfx, src_x, src_y, Gd.GameWin->RPort, dst_x, dst_y, w, h, tags);
}

#endif

#ifdef TARGET_AMIGAOS4

void BlitRect(UWORD src_x, UWORD src_y, UWORD dst_x, UWORD dst_y, UWORD w, UWORD h)
{
	dst_x += Gd.GameWin->BorderLeft;
	dst_y += Gd.GameWin->BorderTop;

	BltBitMapTags(
		BLITA_Source, Gd.GameGfx,
		BLITA_SrcType, BLITT_BITMAP,
		BLITA_SrcX, src_x,
		BLITA_SrcY, src_y,
		BLITA_Dest, Gd.GameWin->RPort,
		BLITA_DestType, BLITT_RASTPORT,
		BLITA_DestX, dst_x,
		BLITA_DestY, dst_y,
		BLITA_Width, w,
		BLITA_Height, h,
		BLITA_UseSrcAlpha, TRUE,
	TAG_END);
}

#endif

#ifdef TARGET_AMIGAOS3

void BlitRect(UWORD src_x, UWORD src_y, UWORD dst_x, UWORD dst_y, UWORD w, UWORD h)
{
	dst_x += Gd.GameWin->BorderLeft;
	dst_y += Gd.GameWin->BorderTop;

	BltBitMapRastPort(Gd.GameGfx, src_x, src_y, Gd.GameWin->RPort, dst_x, dst_y, w, h, 0xC0);
}

#endif

//==============================================================================================================================

void BlitLetter(UBYTE letter, UWORD x, UWORD y, BOOL pressed)
{
	UWORD src_x, src_y, h;

	/* Convert logical block coordinates into pixel coordinates. */

	x *= Gd.BlockWidth;
	y *= Gd.BlockHeight;

	src_x = GetLetterX(letter);
	src_y = GetLetterY(letter, pressed);
	h = Gd.BlockHeight;

	#ifdef TARGET_AMIGAOS3
	if ((letter == '[') || (letter == ']') || (letter == '-')) h >>= 1;
	#endif

	BlitRect(src_x, src_y, x, y, Gd.BlockWidth, h);
}

//==============================================================================================================================
// It clears a logical block in the playfield by blitting appropriate rectangle of background.

void ClearLetter(UWORD x, UWORD y)
{
	UWORD dst_x, dst_y;

	/* Convert logical block coordinates into pixel coordinates. */

	x *= Gd.BlockWidth;
	y *= Gd.BlockHeight;
	dst_x = x + Gd.GameWin->BorderLeft;
	dst_y = y + Gd.GameWin->BorderTop;

	BltBitMapRastPort(Gd.GameGfx, x, y + Gd.BgOffset, Gd.GameWin->RPort, dst_x, dst_y, Gd.BlockWidth, Gd.BlockHeight, 0xC0);
}

//==============================================================================================================================

WORD QuakeShifts[] =  { 1, 2, 3, 2, 1, 0, -1, -2, -1, 0, 1, 0, 100 };

void GenerateAnimBarQuake(UWORD bar_num)
{
	WORD *shift;
	struct AnimEvent *aev;

	for (shift = QuakeShifts; *shift < 100; shift++)
	{
		if (aev = AllocTaskPooled(sizeof(struct AnimEvent)))
		{
			aev->EventKind = ANIM_EVENT_BARQUAKE;
			aev->NextGameState = 0;
			aev->WaitAfter = 0;
			aev->X0 = Gd.BlockWidth;
			aev->Y2 = 1 + 5 * bar_num;
			aev->Y0 = aev->Y2 *Gd.BlockHeight - 3;
			aev->Y1 = *shift + 3;
			AddTail((struct List*)&Gd.AnimQueue, (struct Node*)aev);
		}
	}
}

//==============================================================================================================================

void PerformAnimBarQuake(UWORD x, UWORD y, UWORD delta, UWORD row)
{
	UWORD w, h, r, c, letter, letter_x, letter_y, dest_x, dest_y, *p;

	#ifdef TARGET_MORPHOS
	struct TagItem tags[] = {
		{ BLTBMA_USESOURCEALPHA, TRUE },
		{ TAG_END, 0 }
	};
	#endif

	w = Gd.BlockWidth * 28;
	h = Gd.BlockHeight * 5;

	/* Copy background rectangle to ScratchPad. */

	BltBitMap(Gd.GameGfx, x, y + Gd.BgOffset, Gd.ScratchPad, 0, 0, w, h, 0xC0, ~0, NULL);

	/* Blit the playfield contents */

	p = &Gd.Field[row * FIELD_WIDTH + 1];
	dest_y = delta;

	for (r = 0; r < 5; r++)
	{
		dest_x = 0;

		for (c = 0; c < 28; c++)
		{
			letter = *p++;

			if (letter != ' ')
			{
				UWORD h1 = Gd.BlockHeight;
				letter_x = GetLetterX(letter & FF_LETTERMASK);
				letter_y = GetLetterY(letter & FF_LETTERMASK, letter & FF_CLICKED);

				#if defined TARGET_MORPHOS
				BltBitMapAlpha(Gd.GameGfx, letter_x, letter_y, Gd.ScratchPad, dest_x, dest_y, Gd.BlockWidth, h1, tags);
				#elif defined TARGET_AMIGAOS3
				if ((letter == '[') || (letter == ']') || (letter == '-')) h1 >>= 1;
				BltBitMap(Gd.GameGfx, letter_x, letter_y, Gd.ScratchPad, dest_x, dest_y, Gd.BlockWidth, h1, 0xC0, ~0, NULL);
				#else
				BltBitMapTags(
					BLITA_Source, Gd.GameGfx,
					BLITA_SrcType, BLITT_BITMAP,
					BLITA_SrcX, letter_x,
					BLITA_SrcY, letter_y,
					BLITA_Dest, Gd.ScratchPad,
					BLITA_DestType, BLITT_BITMAP,
					BLITA_DestX, dest_x,
					BLITA_DestY, dest_y,
					BLITA_Width, Gd.BlockWidth,
					BLITA_Height, h1,
					BLITA_UseSrcAlpha, TRUE,
				TAG_END);
				#endif
			}

			dest_x += Gd.BlockWidth;
		}

		p += 2;
		dest_y += Gd.BlockHeight;
	}

	/* Copy ready ScratchPad contents into game window. */

	BltBitMapRastPort(Gd.ScratchPad, 0, 0, Gd.GameWin->RPort, x + Gd.GameWin->BorderLeft,
	 y + Gd.GameWin->BorderTop, w, h, 0xC0);
}


//==============================================================================================================================
// Except of blitting title screen, function installs background fill hook for the window.

void BlitTitleGfx(void)
{
	BltBitMapRastPort(Gd.GameGfx, 0, Gd.BlockHeight << 1, Gd.GameWin->RPort, Gd.GameWin->BorderLeft, Gd.GameWin->BorderTop,
	 Gd.FieldWidth, Gd.FieldHeight, 0xC0);
	InstallLayerHook(Gd.GameWin->RPort->Layer, &LayerHook);

}


//==============================================================================================================================

void BlitBackground(void)
{
	BltBitMapRastPort(Gd.GameGfx, 0, Gd.BgOffset, Gd.GameWin->RPort, Gd.GameWin->BorderLeft, Gd.GameWin->BorderTop,
	 Gd.FieldWidth, Gd.FieldHeight, 0xC0);
}


//==============================================================================================================================
// Uniformly accelerated motion for falling blocks, expressed in units of 1/256 of block height. Accelleration is calculated in
// a way that a block moves by 3 heights in 11 video frames. First row is for block falling by 3 blocks, second - by two blocks
// the third - by one block.

UWORD Falling[3][11] = {
	{ 8, 31, 69, 123, 192, 276, 376, 492, 622, 768, 0 },
	{ 8, 31, 69, 123, 192, 276, 376, 512, 0, 0, 0 },
	{ 8, 31, 69, 123, 192, 256, 0, 0, 0, 0, 0 }
};


// Scrolls one clicked letter (the rest is eliminated already) down. Parameters:
// x - column coordinate
// y - bottom row coordinate, one directly above the bar.
// delta - number of rows to scroll down

void GenerateAnimColumnDown(UWORD x, UWORD y, UWORD delta, UWORD letter)
{
	UWORD *fall, old_y, new_y, off_y;
	UWORD xs;
	BOOL first_in_sequence = TRUE;

	xs = x * Gd.BlockWidth;                                             // playfield pixels, left of area
	off_y = (UWORD)(y - delta) * Gd.BlockHeight;                        // playfield pixels, top of area
	old_y = 0;

	/* Main frame loop. */

	for (fall = Falling[3 - delta]; new_y = *fall; fall++)
	{
		struct AnimEvent *aev;

		if (aev = AllocTaskPooled(sizeof(struct AnimEvent)))
		{
			new_y = (new_y * Gd.BlockHeight >> 8);        // rescale vertical position to pixels
			aev->EventKind = ANIM_EVENT_COLDOWN;
			aev->NextGameState = 0;
			aev->WaitAfter = 0;
			aev->Letter = letter;
			aev->X0 = xs;                                 // horizontal pixel start, inner window
			aev->Y0 = off_y + old_y;                      // vertical pixel start, inner window
			aev->X1 = x;                                  // logical column
			aev->Y1 = new_y - old_y;                      // letter new position related to Y0
			aev->X2 = first_in_sequence;                  // update logical playfield representation
			aev->Y2 = y;                                  // logical row of letter after fall

			if (first_in_sequence) first_in_sequence = FALSE;

			AddTail((struct List*)&Gd.AnimQueue, (struct Node*)aev);
		}

		old_y = new_y;
	}
}

//==============================================================================================================================

#ifdef TARGET_MORPHOS

void PerformAnimColumnDown(UWORD x, UWORD y, UWORD delta, UWORD letter)
{
	UWORD letter_x, letter_y;
	struct TagItem tags[] = {
		{ BLTBMA_USESOURCEALPHA, TRUE },
		{ TAG_END, 0}
	};

	if (delta == 0) return;

	letter_x = GetLetterX(letter);
	letter_y = GetLetterY(letter, TRUE);
	BltBitMap(Gd.GameGfx, x, y + Gd.BgOffset, Gd.ScratchPad, 0, 0, Gd.BlockWidth, Gd.BlockHeight + delta, 0xC0, ~0, NULL);
	BltBitMapAlpha(Gd.GameGfx, letter_x, letter_y, Gd.ScratchPad, 0, delta, Gd.BlockWidth, Gd.BlockHeight, tags);
	BltBitMapRastPort(Gd.ScratchPad, 0, 0, Gd.GameWin->RPort, x + Gd.GameWin->BorderLeft, y + Gd.GameWin->BorderTop,
	 Gd.BlockWidth, Gd.BlockHeight + delta, 0xC0);
}

#endif
#ifdef TARGET_AMIGAOS3

void PerformAnimColumnDown(UWORD x, UWORD y, UWORD delta, UWORD letter)
{
	struct BitMap *bm;
	UWORD letter_x, letter_y;
	ULONG depth;

	if (delta == 0) return;
	depth = GetBitMapAttr(Gd.GameGfx, BMA_DEPTH);

	if (bm = AllocBitMap(Gd.BlockWidth, Gd.BlockHeight + delta, depth, BMF_MINPLANES, Gd.GameScr->RastPort.BitMap))
	{
		letter_x = GetLetterX(letter);
		letter_y = GetLetterY(letter, TRUE);
		BltBitMap(Gd.GameGfx, x, y + Gd.BgOffset, bm, 0, 0, Gd.BlockWidth, Gd.BlockHeight + delta, 0xC0, ~0, NULL);
		BltBitMap(Gd.GameGfx, letter_x, letter_y, bm, 0, delta, Gd.BlockWidth, Gd.BlockHeight, 0xC0, ~0, NULL);
		BltBitMapRastPort(bm, 0, 0, Gd.GameWin->RPort, x + Gd.GameWin->BorderLeft, y + Gd.GameWin->BorderTop,
		 Gd.BlockWidth, Gd.BlockHeight + delta, 0xC0);
		FreeBitMap(bm);
	}
}

#endif
#ifdef TARGET_AMIGAOS4

void PerformAnimColumnDown(UWORD x, UWORD y, UWORD delta, UWORD letter)
{
	struct BitMap *bm;
	struct TagItem tags[] = {
		{ BLITA_Source, Gd.GameGfx },
		{ BLITA_SrcType, BLITT_BITMAP },
		{ BLITA_SrcX, x },
		{ BLITA_SrcY, y + Gd.BgOffset },
		{ BLITA_Dest, 0 },
		{ BLITA_DestType, BLITT_BITMAP },
		{ BLITA_DestX, 0 },
		{ BLITA_DestY, 0 },
		{ BLITA_Width, Gd.BlockWidth },
		{ BLITA_Height, Gd.BlockHeight + delta },
		{ BLITA_UseSrcAlpha, FALSE },
		{ TAG_END, 0 }
	};

	if (delta == 0) return;

	if (bm = AllocBitMap(Gd.BlockWidth, Gd.BlockHeight + delta, 32, BMF_DISPLAYABLE, Gd.GameScr->RastPort.BitMap))
	{
		tags[4].ti_Data = (IPTR)bm;

		BltBitMapTagList(tags);

		tags[2].ti_Data = GetLetterX(letter);
		tags[3].ti_Data = GetLetterY(letter, TRUE);
		tags[7].ti_Data = delta;
		tags[9].ti_Data = Gd.BlockHeight;
		tags[10].ti_Data = TRUE;

		BltBitMapTagList(tags);

		tags[0].ti_Data = (IPTR)bm;
		tags[2].ti_Data = 0;
		tags[3].ti_Data = 0;
		tags[4].ti_Data = (IPTR)Gd.GameWin->RPort;
		tags[5].ti_Data = BLITT_RASTPORT;
		tags[6].ti_Data = x + Gd.GameWin->BorderLeft;
		tags[7].ti_Data = y + Gd.GameWin->BorderTop;
		tags[9].ti_Data = Gd.BlockHeight + delta;
		tags[10].ti_Data = FALSE;

		BltBitMapTagList(tags);

		FreeBitMap(bm);
	}
}

#endif

//==============================================================================================================================
// Pushes sequence of AnimEvents to AnimQueue, which scrolls up the title screen, replacing it with background.
//
// The table contains positions of "curtain" moved up with uniformly accelerated motion. Positions are normalized to window
// height being 256. Then having the real window height, position for each frame is calculated as win_h * table[frame] >> 8.
//
// After the curtain is up, 12 wait frames are inserted.


UWORD CurtainMov[24] = 
{ 
	2, 4, 7, 10, 15, 20, 26, 33, 41, 50, 59, 69, 80, 92, 105, 118, 133, 148, 164, 181, 198, 217, 236, 256
};


void AnimCurtainUp(void)
{
	struct AnimEvent *aev;
	UWORD i, newpos, oldpos = 0;
	ULONG v;

	for (i = 0; i < 25; i++)
	{
		if (aev = AllocTaskPooled(sizeof(struct AnimEvent)))
		{
			if (i < 24)
			{
				aev->EventKind = ANIM_EVENT_SCROLL;
				aev->WaitAfter = 0;
				aev->NextGameState = 0;
				newpos = CurtainMov[i];
				v = newpos * Gd.FieldHeight >> 8;
				aev->X0 = Gd.GameWin->BorderLeft;
				aev->X1 = aev->X0 + Gd.FieldWidth - 1;
				aev->Y0 = Gd.GameWin->BorderTop;
				aev->Y1 = aev->Y0 + Gd.FieldHeight - oldpos - 1;
				aev->X2 = 0;
				aev->Y2 = v - oldpos;
				oldpos = v;
			}
			else
			{
				aev->EventKind = ANIM_EVENT_WAIT;
				aev->WaitAfter = 12;
				aev->NextGameState = GAME_SOLVING_PUZZLE;
			}

			AddTail((struct List*)&Gd.AnimQueue, (struct Node*)aev);
		}
	}
}
