/* 
  Crazy Crane
  Grzegorz Kraszewski, 2013.
  Main code file.
*/


#include "main.h"
#include "gfx.h"
#include "game.h"

#include <proto/intuition.h>


struct Library
	*IntuitionBase,
	*DataTypesBase,
	*GfxBase,
	*LayersBase,
	*UtilityBase;

#ifdef TARGET_AMIGAOS3
#define TIMER_UNIT UNIT_ECLOCK
#endif

#ifdef TARGET_MORPHOS    /* Reggae! CGX! */
#define TIMER_UNIT UNIT_CPUCLOCK
struct Library *MultimediaBase;
struct Library *CyberGfxBase;
#endif  /* TARGET_MORPHOS */

#ifdef TARGET_AMIGAOS4
#define TIMER_UNIT UNIT_ECLOCK
struct IntuitionIFace *IIntuition;
struct DataTypesIFace *IDataTypes;
struct GraphicsIFace *IGraphics;
struct LayersIFace *ILayers;
struct UtilityIFace *IUtility;
#endif  /* TARGET_AMIGAOS4 */


struct GameData Gd;


LONG open_libs(void);
void close_libs(void);



LONG Main(UNUSED struct WBStartup *wbs)
{
	LONG result = RETURN_OK;

	if ((result = open_libs()) == RETURN_OK)
	{
		if (GraphicsLoad("PROGDIR:graphics/" DEFAULT_GFX_FILE))
		{
			Gd.FieldWidth = Gd.BlockWidth * FIELD_WIDTH;
			Gd.FieldHeight = Gd.BlockHeight * FIELD_HEIGHT;

			if (Gd.GameWin = OpenWindowTags(NULL,
				WA_InnerHeight, Gd.FieldHeight,
				WA_InnerWidth, Gd.FieldWidth,
				WA_DragBar, TRUE,
				WA_Title, (IPTR)"CrazyCrane",
				WA_CloseGadget, TRUE,
				WA_DepthGadget, TRUE,
				WA_IDCMP, IDCMP_CLOSEWINDOW | IDCMP_MOUSEBUTTONS
			))
			{
				TheGame();
				CloseWindow(Gd.GameWin);
			}

			GraphicsDispose();
		}
	}
	else Printf("Nie dao si otworzy potrzebnych bibliotek, system wydaje si by niekompletny...\n");

	close_libs();

	return result;
}



LONG open_libs(void)
{
	if (!(IntuitionBase = OpenLibrary("intuition.library", 39))) return RETURN_FAIL;
	if (!(DataTypesBase = OpenLibrary("datatypes.library", 39))) return RETURN_FAIL;
	if (!(GfxBase = OpenLibrary("graphics.library", 39))) return RETURN_FAIL;
	if (!(LayersBase = OpenLibrary("layers.library", 39))) return RETURN_FAIL;
	if (!(UtilityBase = OpenLibrary("utility.library", 39))) return RETURN_FAIL;

	#ifdef TARGET_AMIGAOS4
	if (!(IIntuition = (struct IntuitionIFace*)GetInterface(IntuitionBase, "main", 1, NULL))) return RETURN_FAIL;
	if (!(IDataTypes = (struct DataTypesIFace*)GetInterface(DataTypesBase, "main", 1, NULL))) return RETURN_FAIL;
	if (!(IGraphics = (struct GraphicsIFace*)GetInterface(GfxBase, "main", 1, NULL))) return RETURN_FAIL;
	if (!(ILayers = (struct LayersIFace*)GetInterface(LayersBase, "main", 1, NULL))) return RETURN_FAIL;
	if (!(IUtility = (struct UtilityIFace*)GetInterface(UtilityBase, "main", 1, NULL))) return RETURN_FAIL;
	#endif  /* TARGET_AMIGAOS4 */

	#ifdef TARGET_MORPHOS
	if (!(MultimediaBase = OpenLibrary("multimedia/multimedia.class", 55))) return RETURN_FAIL;
	if (!(CyberGfxBase = OpenLibrary("cybergraphics.library", 50))) return RETURN_FAIL;
	#endif  /* TARGET_MORPHOS */

	return RETURN_OK;
}



void close_libs(void)
{
	#ifdef TARGET_MORPHOS
	if (CyberGfxBase) CloseLibrary(CyberGfxBase);
	if (MultimediaBase) CloseLibrary(MultimediaBase);
	#endif  /* TARGET_MORPHOS */

	#ifdef TARGET_AMIGAOS4
	if (IUtility) DropInterface((struct Interface*)IUtility);
	if (ILayers) DropInterface((struct Interface*)ILayers);
	if (IGraphics) DropInterface((struct Interface*)IGraphics);
	if (IIntuition) DropInterface((struct Interface*)IIntuition);
	if (IDataTypes) DropInterface((struct Interface*)IDataTypes);
	#endif  /* TARGET_AMIGAOS4 */

	if (UtilityBase) CloseLibrary(UtilityBase);
	if (LayersBase) CloseLibrary(LayersBase);
	if (GfxBase) CloseLibrary(GfxBase);
	if (DataTypesBase) CloseLibrary(DataTypesBase);
	if (IntuitionBase) CloseLibrary(IntuitionBase);
}


#ifdef TARGET_AMIGAOS3

APTR AllocVecPooled(APTR pool, long memsize)
{
	UBYTE *baseaddr = NULL;

	memsize += 4;

	if (baseaddr = AllocPooled(pool, (ULONG)memsize))
	{
		*(ULONG*)baseaddr = memsize;
		return (APTR)(baseaddr + 4);
	}

	return NULL;
}


void FreeVecPooled(APTR pool, APTR memory)
{
	UBYTE *baseaddr = (UBYTE*)memory - 4;

	FreePooled(pool, baseaddr, *(ULONG*)baseaddr);
}

#endif


/* Auxiliary functions. */

LONG StrLen(CONST_STRPTR s)
{
	CONST_STRPTR v = s;

	while (*v) v++;
	return (LONG)(v - s);
}


BOOL StrEqu(CONST_STRPTR s1, CONST_STRPTR s2)
{
	while (*s1 && (*s1 == *s2)) { s1++; s2++; }
	if (*s1 == *s2) return TRUE;
	else return FALSE;
}


STRPTR StrCopy(CONST_STRPTR s, STRPTR d)
{
	while (*d++ = *s++);
	return (--d);
}



STRPTR StrNew(CONST_STRPTR s)
{
	STRPTR n = NULL;

	if (n = AllocVecTaskPooled(StrLen(s) + 1))
	{
		StrCopy(s, n);
	}

	return n;
}



void StrFree(CONST_STRPTR s)
{
	FreeVecTaskPooled((APTR)s);
}


