/* 
  Crazy Crane
  Grzegorz Kraszewski, 2013.
  Main header file.
*/


#define __NOLIBBASE__

#ifdef TARGET_MORPHOS
typedef char* STRPTR;
typedef const char* CONST_STRPTR;
#define STRPTR_TYPEDEF
#else
typedef unsigned int IPTR;
#endif  /* TARGET_MORPHOS */

#define UNUSED __attribute__((unused))


#ifdef TARGET_MORPHOS
#define SYSTEM "MorphOS"
#define DEFAULT_GFX_FILE "crazycrane24.png"
#elif defined TARGET_AMIGAOS4
#define SYSTEM "AmigaOS 4"
#define DEFAULT_GFX_FILE "crazycrane24.png"
#else
#define SYSTEM "AmigaOS 3"
#define DEFAULT_GFX_FILE "crazycrane.iff"
#endif

#define FIELD_WIDTH   30
#define FIELD_HEIGHT  17
#define POINT_INTERVAL 50       // How often to decrease points by 1 in game heartbeats (1/50 s).

#define VSTRING "$VER: CrazyCrane 0.5 (22.01.2013) Leon, Krashan, Mccnex\r\n"
#define SCREEN_TITLE "Crazy Crane 0.5 (22.01.2013) Leon, Krashan, Mccnex"

#include <proto/exec.h>
#include <proto/dos.h>
#include <workbench/startup.h>
#include <intuition/intuition.h>

#include "counter.h"


#define ITERATE_LIST(listptr, type, var) \
for (var = (type)((struct MinList*)listptr)->mlh_Head; \
 ((struct MinNode*)var)->mln_Succ; \
 var = (type)((struct MinNode*)var)->mln_Succ)

#define INIT_LIST(listptr) \
((struct MinList*)listptr)->mlh_Head = (struct MinNode*)&((struct MinList*)listptr)->mlh_Tail; \
((struct MinList*)listptr)->mlh_Tail = NULL; \
((struct MinList*)listptr)->mlh_TailPred = (struct MinNode*)&((struct MinList*)listptr)->mlh_Head;


extern LONG Main(struct WBStartup *wbmsg);


extern struct Library
	*SysBase,
	*DOSBase,
	*IntuitionBase,
	*DataTypesBase,
	*GfxBase,
	*LayersBase,
	*UtilityBase;

#ifdef TARGET_MORPHOS
extern struct Library *MultimediaBase;
extern struct Library *CyberGfxBase;
#endif

#ifdef TARGET_AMIGAOS4
extern struct IntuitionIFace *IIntuition;
extern struct DataTypesIFace *IDataTypes;
extern struct LayersIFace *ILayers;
extern struct UtilityIFace *IUtility;
#endif  /* TARGET_AMIGAOS4 */


struct AnimEvent
{
	struct MinNode Node;                       // For queueing.
	LONG Points;                               // Points added when the event starts (triggers point update).
	UWORD WaitAfter;                           // How many game ticks to wait after performing event.
	UWORD EventKind;                           // Kind of event.
	UWORD NextGameState;                       // Change if not zero.
	UWORD Letter;                              // Letter ASCII for ANIM_EVENT_COLDOWN.
	UWORD X0;                                  // Parameters of event.
	UWORD Y0;
	UWORD X1;
	UWORD Y1;
	UWORD X2;
	UWORD Y2;
};


#define ANIM_EVENT_SCROLL                    1 // Scrolling with automatic background reconstruction
                                               //   x0, y0, x1, y1 define scroll area,
                                               //   x2, y2 are scroll deltas.

#define ANIM_EVENT_WAIT                      2 // Just waiting, event parameters are ignored.

#define ANIM_EVENT_CLEAR                     3 // Clear area to background.
                                               //   x0, y0 logical block coordinates

#define ANIM_EVENT_COLDOWN                   4 // Scrolls column down with a single clicked letter.
                                               //   x0 - logical column
                                               //   y0 - top of scrolled area
                                               //   x1 - abused for scrolled letter
                                               //   y1 - scroll delta
                                               //   x2 - boolean, update playfield when TRUE
                                               //   y2 - logical row of letter after fall

#define ANIM_EVENT_BARQUAKE                  5 // Does bar quake on one phrase row.

struct GameData
{
	struct BitMap *GameGfx;                    // Bitmap containing all the game graphics. Friend of screen's bitmap.
	struct BitMap *ScratchPad;                 // Used for animations as temporary bitmap.
	struct Window *GameWin;                    // Game window.
	struct Screen *GameScr;                    // Screen on which the game window is opened.
	UWORD BlockWidth;                          // Letter block width in pixels.
	UWORD BlockHeight;                         // Letter block height in pixels.
	UWORD FieldWidth;                          // Pixel width of the playfield.
	UWORD FieldHeight;                         // Pixel height of the playfield.
	UWORD BgOffset;                            // Vertical offset of background image in GameGfx.
	UBYTE LetterTable[39];                     // Transforms ASCII code to letter block index in GameGfx.
	UWORD Field[FIELD_WIDTH * FIELD_HEIGHT];   // Game field (ASCII codes + flags in the upper byte).
	struct Counter PCounter;                   // Point counter stuff.
	UBYTE LettersInRow[3];                     // Number of letters in each of three rows.
	STRPTR OriginalString;                     // String to guess.
	STRPTR ClickedString;                      // String with clicked letters.
	ULONG Points;                              // Points collected by player.
	ULONG PointsCountDown;                     // Determines when points are decreased.
	UWORD AnimCounter;                         // Used for animations.
	struct MinList AnimQueue;                  // List of anim events to perform.
	UWORD DelayedState;                        // Game state to be set after current anim event delay ends.
	struct MinList Questions;                  // List of questions.
	ULONG QuestionCount;                       // Number of not yet answered questions in the set.
	char WinTitleBuf[128];                     // Buffer for window title.
};

/* Game states */

#define GAME_TITLE_SCREEN            1    // displays title screen
#define GAME_CURTAIN_UP              2    // curtain up animation
#define GAME_SOLVING_PUZZLE          3    // puzzle is displayed and is being solved


/* memory handling */

#ifndef TARGET_MORPHOS

extern APTR CrazyPool;         // memory pool for the game

#define AllocTaskPooled(s) AllocPooled(CrazyPool, s)
#define AllocVecTaskPooled(s) AllocVecPooled(CrazyPool, s)
#define FreeTaskPooled(a, s) FreePooled(CrazyPool, a, s)
#define FreeVecTaskPooled(a) FreeVecPooled(CrazyPool, a)

#ifdef TARGET_AMIGAOS3

APTR AllocVecPooled(APTR pool, long memsize);
void FreeVecPooled(APTR pool, APTR memory);

#endif
#endif  /* TARGET_MORPHOS */


/* Auxiliary functions. */

LONG StrLen(CONST_STRPTR s);
BOOL StrEqu(CONST_STRPTR s1, CONST_STRPTR s2);
STRPTR StrCopy(CONST_STRPTR s, STRPTR d);
STRPTR StrNew(CONST_STRPTR s);
void StrFree(CONST_STRPTR s);
