/* 
  Crazy Crane - random number generator
  Enthropy source:
  On MorphOS I use random.library.
  On AmigaOS 4 and AmigaOS 3 EClock is used.
*/

#include "main.h"
#include "random.h"
#include <proto/exec.h>

ULONG ShiftReg;     // generator state is stored here

//==============================================================================================================================

#ifdef TARGET_AMIGAOS3

#include <proto/timer.h>
#include <devices/timer.h>

void RandomSeed(void)
{
	struct IORequest *treq;
	struct MsgPort *port;

	if (port = CreateMsgPort())
	{
		if (treq = CreateIORequest(port, sizeof(struct IORequest)))
		{
			if (OpenDevice("timer.device", UNIT_VBLANK, treq, 0) == 0)
			{
				struct Library *TimerBase = &treq->io_Device->dd_Library;
				struct EClockVal eclk;

				ReadEClock(&eclk);
				ShiftReg = ((eclk.ev_lo & 0xFFFF) | (eclk.ev_lo << 16)) ^ 0x96A5C31E;
				CloseDevice(treq);
			}
			DeleteIORequest(treq);
		}
		DeleteMsgPort(port);
	}
}

#elif defined TARGET_AMIGAOS4

#define __USE_OLD_TIMEVAL__
#include <proto/timer.h>
#include <devices/timer.h>

void RandomSeed(void)
{
	struct IORequest *treq;
	struct MsgPort *port;

	if (port = CreateMsgPort())
	{
		if (treq = CreateIORequest(port, sizeof(struct IORequest)))
		{
			if (OpenDevice("timer.device", UNIT_VBLANK, treq, 0) == 0)
			{
				struct Library *TimerBase = &treq->io_Device->dd_Library;
				struct EClockVal eclk;
				struct TimerIFace *ITimer;
				
				if (ITimer = (struct TimerIFace*)GetInterface(TimerBase, "main", 1, NULL))
				{
					ReadEClock(&eclk);
					ShiftReg = ((eclk.ev_lo & 0xFFFF) | (eclk.ev_lo << 16)) ^ 0x96A5C31E;
					DropInterface((struct Interface*)ITimer);
				}
				CloseDevice(treq);
			}
			DeleteIORequest(treq);
		}
		DeleteMsgPort(port);
	}
}

#else

#include <proto/random.h>

void RandomSeed(void)
{
	struct Library *RandomBase;

	if (RandomBase = OpenLibrary("random.library", 1))
	{
		RandomStir();
		ShiftReg = Random();
		CloseLibrary(RandomBase);
	}
}

#endif

//==============================================================================================================================

/* This is simple Linear Feedback Shift Register with 2^32 period.             */
/* It is enough for game or audio noise generation, but do not use it for      */
/* cryptography or simulations!                                                */


UBYTE RandomUByte(void)
{
	UBYTE r = 0;
	UBYTE i;
	ULONG f;

	for (i = 0; i < 8; i++)
	{
		f = ShiftReg;
		r <<= 1;
		if (ShiftReg & 0x80000000) r |= 1;
		ShiftReg <<= 1;
		ShiftReg |= ((f >> 30) ^ (f >> 27)) & 1;
	}

	return r;
}


UBYTE RandomScaled(UBYTE m)
{
	UBYTE s = 0, x, u;

	/*-----------------------------------------------------------------------------------------------*/
	/* Calculate shift for limiting random range to the nearest power of two higher of equal to 'm'. */
	/*-----------------------------------------------------------------------------------------------*/

	if (m == 0) return 0;
	u = m;

	while ((m & 0x80) == 0)
	{
		m <<= 1;
		s++;
	}

	/*------------------------------------------------------------------------------------------------*/
	/* Keep trying until scaled random number fits the range. Statistically it takes 1.5 of tries for */
	/* the worst case of 'm' being a power of two.                                                    */
	/*------------------------------------------------------------------------------------------------*/

	do
	{
		x = RandomUByte() >> s;
	}
	while (x > u);

	return x;
}
